@@ -1,7 +1,8 @@
 import re
+from collections.abc import Generator, Iterator
 from itertools import cycle
 from pathlib import Path
-from typing import Generator, Optional
+from typing import Any, Optional, TypeVar
 
 import typer
 from rich.progress import Progress, track
@@ -21,7 +22,7 @@ from . import utils
 app = typer.Typer()
 
 
-def validate_name(name: str):
+def validate_name(name: str) -> str:
     match = re.match(r"^[\w\-_]*$", name)
     if match is None:
         raise typer.BadParameter("Platform name must be slug-like.")
@@ -29,19 +30,19 @@ def validate_name(name: str):
         return name
 
 
-def validate_dsn(dsn: str | None):
+def validate_dsn(dsn: str | None) -> str | None:
     if dsn is None:
         return None
     match = re.match(r"^(sqlite|postgresql\+psycopg|https|http)(\:\/\/)", dsn)
     if match is None:
         raise typer.BadParameter(
-            "Platform dsn must be a valid URl or database connection string."
+            "Platform dsn must be a valid URL or database connection string."
         )
     else:
         return dsn
 
 
-def prompt_sqlite_dsn(name: str):
+def prompt_sqlite_dsn(name: str) -> str:
     path = sqlite.get_database_path(name)
     dsn = sqlite.get_dsn(path)
     if path.exists():
@@ -57,6 +58,7 @@ def prompt_sqlite_dsn(name: str):
             f"No file at the standard filesystem location for name '{name}' exists. "
             "Do you want to create a new database?"
         ):
+            utils.echo("Creating the database and running migrations...")
             alembic.upgrade_database(dsn, "head")
             return dsn
         else:
@@ -75,7 +77,7 @@ def add(
         help="Data source name. Can be a http(s) URL or a database connection string.",
         callback=validate_dsn,
     ),
-):
+) -> None:
     try:
         settings.toml.get_platform(name)
         raise typer.BadParameter(
@@ -95,11 +97,11 @@ def add(
     utils.good("\nPlatform added successfully.")
 
 
-def prompt_sqlite_removal(dsn: str):
+def prompt_sqlite_removal(dsn: str) -> None:
     path = Path(dsn.replace("sqlite://", ""))
     path_str = typer.style(path, fg=typer.colors.CYAN)
     if typer.confirm(
-        "Do you want to remove the associated database file at " f"{path_str} as well?"  # type: ignore
+        f"Do you want to remove the associated database file at {path_str} as well?"
     ):
         path.unlink()
         utils.echo("\nDatabase file deleted.")
@@ -112,7 +114,7 @@ def remove(
     name: str = typer.Argument(
         ..., help="The string identifier of the platform to remove."
     ),
-):
+) -> None:
     try:
         platform = settings.toml.get_platform(name)
     except PlatformNotFound:
@@ -127,7 +129,7 @@ def remove(
         settings.toml.remove_platform(name)
 
 
-def tabulate_toml_platforms(platforms: list[TomlPlatformInfo]):
+def tabulate_toml_platforms(platforms: list[TomlPlatformInfo]) -> None:
     toml_path_str = typer.style(settings.toml.path, fg=typer.colors.CYAN)
     utils.echo(f"\nPlatforms registered in '{toml_path_str}'")
     if len(platforms):
@@ -140,7 +142,7 @@ def tabulate_toml_platforms(platforms: list[TomlPlatformInfo]):
 
 def tabulate_manager_platforms(
     platforms: list[ManagerPlatformInfo],
-):
+) -> None:
     manager_url_str = typer.style(settings.manager.url, fg=typer.colors.CYAN)
     utils.echo(f"\nPlatforms accessible via '{manager_url_str}'")
     utils.echo("\nName".ljust(21) + "Access".ljust(10) + "Notice")
@@ -154,7 +156,7 @@ def tabulate_manager_platforms(
 
 
 @app.command("list", help="Lists all registered platforms.")
-def list_():
+def list_() -> None:
     tabulate_toml_platforms(settings.toml.list_platforms())
     if settings.manager is not None:
         tabulate_manager_platforms(settings.manager.list_platforms())
@@ -166,7 +168,8 @@ def list_():
         "revision."
     )
 )
-def upgrade():
+def upgrade() -> None:
+    platform_list: list[ManagerPlatformInfo] | list[TomlPlatformInfo]
     if settings.managed:
         utils.echo(
             f"Establishing self-signed admin connection to '{settings.manager_url}'."
@@ -193,6 +196,40 @@ def upgrade():
                 utils.echo(f"Skipping '{p.name}' because of an error: {str(e)}")
 
 
+@app.command(help=("Downgrades all database platforms to the given revision."))
+def downgrade(
+    revision: str = typer.Argument(
+        ...,
+        help="The revision to downgrade to.",
+    ),
+) -> None:
+    platform_list: list[ManagerPlatformInfo] | list[TomlPlatformInfo]
+    if settings.managed:
+        utils.echo(
+            f"Establishing self-signed admin connection to '{settings.manager_url}'."
+        )
+        manager_conf = ManagerConfig(
+            str(settings.manager_url),
+            SelfSignedAuth(settings.secret_hs256),
+            remote=False,
+        )
+        platform_list = manager_conf.list_platforms()
+    else:
+        platform_list = settings.toml.list_platforms()
+
+    for p in platform_list:
+        if p.dsn.startswith("http"):
+            # This should probably never happen unless the manager registers an
+            # external rest platform.
+            utils.echo(f"Skipping '{p.name}' because it is a REST platform.")
+        else:
+            utils.echo(f"Downgrading platform '{p.name}' with dsn '{p.dsn}'...")
+            try:
+                alembic.downgrade_database(p.dsn, revision)
+            except OperationalError as e:
+                utils.echo(f"Skipping '{p.name}' because of an error: {str(e)}")
+
+
 @app.command(
     help=(
         "Stamps all database platforms from your local toml file with the given "
@@ -232,7 +269,7 @@ def generate(
     num_datapoints: int = typer.Option(
         30_000, "--datapoints", help="Number of mock datapoints to generate."
     ),
-):
+) -> None:
     try:
         platform = Platform(platform_name)
     except PlatformNotFound:
@@ -270,7 +307,12 @@ def generate(
         utils.good("Done!")
 
 
-def create_cycle(generator: Generator, name: str, total: int):
+T = TypeVar("T")
+
+
+def create_cycle(
+    generator: Generator[T, Any, None], name: str, total: int
+) -> Iterator[T]:
     return cycle(
         [
             m
@@ -283,7 +325,7 @@ def create_cycle(generator: Generator, name: str, total: int):
     )
 
 
-def generate_data(generator: MockDataGenerator):
+def generate_data(generator: MockDataGenerator) -> None:
     model_names = create_cycle(
         generator.yield_model_names(), "Model", generator.num_models
     )
@@ -301,7 +343,7 @@ def generate_data(generator: MockDataGenerator):
             progress.advance(task, len(df))
 
 
-def _shorten(value: str, length: int):
+def _shorten(value: str, length: int) -> str:
     """Shorten and adjust a string to a given length adding `...` if necessary"""
     if len(value) > length - 4:
         value = value[: length - 4] + "..."
