@@ -1,4 +1,5 @@
 from typing import Protocol
+
 import pydantic
 
 
@@ -8,8 +9,6 @@ class PlatformInfo(pydantic.BaseModel):
 
 
 class Config(Protocol):
-    def list_platforms(self) -> list[PlatformInfo]:
-        ...
+    def list_platforms(self) -> list[PlatformInfo]: ...
 
-    def get_platform(self, key: str) -> PlatformInfo:
-        ...
+    def get_platform(self, key: str) -> PlatformInfo: ...
