@@ -1,3 +1,4 @@
+from collections.abc import Iterable
 from datetime import datetime
 
 import pandas as pd
@@ -29,25 +30,25 @@ class Unit(BaseModelFacade):
     def created_by(self) -> str | None:
         return self._model.created_by
 
-    def delete(self):
+    def delete(self) -> None:
         self.backend.units.delete(self._model.id)
 
     @property
-    def docs(self):
+    def docs(self) -> str | None:
         try:
             return self.backend.units.docs.get(self.id).description
         except DocsModel.NotFound:
             return None
 
     @docs.setter
-    def docs(self, description):
+    def docs(self, description: str | None) -> None:
         if description is None:
             self.backend.units.docs.delete(self.id)
         else:
             self.backend.units.docs.set(self.id, description)
 
     @docs.deleter
-    def docs(self):
+    def docs(self) -> None:
         try:
             self.backend.units.docs.delete(self.id)
         # TODO: silently failing
@@ -59,10 +60,7 @@ class Unit(BaseModelFacade):
 
 
 class UnitRepository(BaseFacade):
-    def create(
-        self,
-        name: str,
-    ) -> Unit:
+    def create(self, name: str) -> Unit:
         if name != "" and name.strip() == "":
             raise ValueError("Using a space-only unit name is not allowed.")
         if name == "dimensionless":
@@ -72,7 +70,7 @@ class UnitRepository(BaseFacade):
         model = self.backend.units.create(name)
         return Unit(_backend=self.backend, _model=model)
 
-    def delete(self, x: Unit | int | str):
+    def delete(self, x: Unit | int | str) -> None:
         if isinstance(x, Unit):
             id = x.id
         elif isinstance(x, int):
@@ -97,9 +95,8 @@ class UnitRepository(BaseFacade):
         return self.backend.units.tabulate(name=name)
 
     def _get_unit_id(self, unit: str) -> int | None:
-        if unit is None:
-            return None
-        elif isinstance(unit, str):
+        # NOTE leaving this check for users without mypy
+        if isinstance(unit, str):
             obj = self.backend.units.get(unit)
             return obj.id
         else:
@@ -135,3 +132,13 @@ class UnitRepository(BaseFacade):
             return None
         except DocsModel.NotFound:
             return None
+
+    def list_docs(
+        self, id: int | None = None, id__in: Iterable[int] | None = None
+    ) -> Iterable[str]:
+        return [
+            item.description
+            for item in self.backend.units.docs.list(
+                dimension_id=id, dimension_id__in=id__in
+            )
+        ]
