@@ -3,6 +3,7 @@ This module holds a shared datastructure and interface for normalization
 between the database and api data models and repositories.
 """
 
+from .annotations import HasNameFilter
 from .base import (
     BaseMeta,
     BaseModel,
@@ -15,6 +16,7 @@ from .base import (
     Retriever,
     Tabulator,
 )
+from .checkpoint import Checkpoint, CheckpointRepository
 from .docs import Docs, DocsRepository
 from .iamc import (  # AnnualDataPoint,; SubAnnualDataPoint,; CategoricalDataPoint,
     DataPoint,
@@ -29,6 +31,7 @@ from .iamc import (  # AnnualDataPoint,; SubAnnualDataPoint,; CategoricalDataPoi
 from .meta import MetaValue, RunMetaEntry, RunMetaEntryRepository, StrictMetaValue
 from .model import Model, ModelRepository
 from .optimization import (
+    BackendBaseRepository,
     Equation,
     EquationRepository,
     IndexSet,
