@@ -1,18 +1,17 @@
+from collections.abc import Iterable
 from typing import Protocol
 
-from ixmp4.data import types
-
 from . import base
 
 
 class Docs(base.BaseModel, Protocol):
     """Abstract documentation model for one object of any dimension."""
 
-    description: types.String
+    description: str
     "Description of the dimension object."
-    dimension__id: types.Integer
+    dimension__id: int
     "Foreign unique integer id of the object in the dimension's table."
-    dimension: types.Mapped
+    dimension: base.BaseModel
     "The documented object."
 
     # This doesn't work since each dimension has a different self.dimension object as
@@ -23,13 +22,7 @@ class Docs(base.BaseModel, Protocol):
     #    )
 
 
-# TODO: adjust all type hints once things work
-class DocsRepository(
-    base.Retriever,
-    base.Deleter,
-    base.Enumerator,
-    Protocol,
-):
+class DocsRepository(base.Retriever, base.Deleter, base.Enumerator, Protocol):
     def get(self, dimension_id: int) -> Docs:
         """Retrieve the documentation of an object of any dimension.
 
@@ -89,14 +82,18 @@ class DocsRepository(
         self,
         *,
         dimension_id: int | None = None,
+        dimension_id__in: Iterable[int] | None = None,
     ) -> list[Docs]:
         """Lists documentations.
 
         Parameters
         ----------
-        dimension_id : int
+        dimension_id : int or None
             The id of an object in any dimension. If supplied only one result will be
             returned.
+        dimension_id__in : list[int] or None
+            A list of ids of objects in any one dimension. If supplied, only these
+            objects will be returned.
 
         Returns
         -------
