@@ -1,23 +1,24 @@
-from typing import Protocol
+from typing import TYPE_CHECKING, Protocol
+
+if TYPE_CHECKING:
+    from .. import Unit, Variable
 
 import pandas as pd
 
-from ixmp4.data import types
-
 from .. import base
 
 
 class Measurand(base.BaseModel, Protocol):
     """Measurand data model."""
 
-    variable__id: types.Integer
+    variable__id: int
     "Foreign unique integer id of a variable."
-    variable: types.Mapped
+    variable: "Variable"
     "Associated variable."
 
-    unit__id: types.Integer
+    unit__id: int
     "Foreign unique integer id of a unit."
-    unit: types.Mapped
+    unit: "Unit"
     "Associated unit."
 
     def __str__(self) -> str:
@@ -40,10 +41,6 @@ class MeasurandRepository(
         except Measurand.NotFound:
             return self.create(variable_name, unit__id)
 
-    def list(
-        self,
-    ) -> list[Measurand]: ...
+    def list(self) -> list[Measurand]: ...
 
-    def tabulate(
-        self,
-    ) -> pd.DataFrame: ...
+    def tabulate(self) -> pd.DataFrame: ...
