@@ -1,12 +1,15 @@
+from collections.abc import Iterable
 from enum import Enum
 from typing import ClassVar, Protocol
 
 import pandas as pd
 from pydantic import StrictBool, StrictFloat, StrictInt, StrictStr
 
-from ixmp4.data import types
+# TODO Import this from typing when dropping Python 3.11
+from typing_extensions import Unpack
 
 from . import base
+from .annotations import HasIdFilter, HasRunFilter, HasRunIdFilter, PrimitiveTypes
 
 # as long as all of these are `Strict` the order does not matter
 StrictMetaValue = StrictBool | StrictInt | StrictFloat | StrictStr
@@ -23,19 +26,24 @@ class RunMetaEntry(base.BaseModel, Protocol):
         BOOL = "BOOL"
 
         @classmethod
-        def from_pytype(cls, type_):
+        def from_pytype(cls, type_: type) -> str:
             return RunMetaEntry._type_map[type_]
 
-    run__id: types.Integer
+    run__id: int
     "Foreign unique integer id of a run."
-    key: types.String
+    key: str
     "Key for the entry. Unique for each `run__id`."
-    type: types.String
+    dtype: str
     "Datatype of the entry's value."
-    value: types.Integer | types.Float | types.Integer | types.Boolean
+    value: PrimitiveTypes
     "Value of the entry."
 
-    _type_map: ClassVar[dict] = {
+    value_int: int | None
+    value_str: str | None
+    value_float: float | None
+    value_bool: bool | None
+
+    _type_map: ClassVar[dict[type, str]] = {
         int: Type.INT,
         str: Type.STR,
         float: Type.FLOAT,
@@ -47,6 +55,41 @@ class RunMetaEntry(base.BaseModel, Protocol):
             key={self.key} value={self.value}"
 
 
+class EnumerateKwargs(HasIdFilter, HasRunIdFilter, total=False):
+    dtype: str
+    dtype__in: Iterable[str]
+    dtype__like: str
+    dtype__ilike: str
+    dtype__notlike: str
+    dtype__notilike: str
+    key: str
+    key__in: Iterable[str]
+    key__like: str
+    key__ilike: str
+    key__notlike: str
+    key__notilike: str
+    value_int: int
+    value_int_id__in: Iterable[int]
+    value_int_id__gt: int
+    value_int_id__lt: int
+    value_int_id__gte: int
+    value_int_id__lte: int
+    value_str: str
+    value_str__in: Iterable[str]
+    value_str__like: str
+    value_str__ilike: str
+    value_str__notlike: str
+    value_str__notilike: str
+    value_float: float
+    value_float_id__in: Iterable[float]
+    value_float_id__gt: float
+    value_float_id__lt: float
+    value_float_id__gte: float
+    value_float_id__lte: float
+    value_bool: bool
+    run: HasRunFilter
+
+
 class RunMetaEntryRepository(
     base.Creator,
     base.Retriever,
@@ -56,12 +99,7 @@ class RunMetaEntryRepository(
     base.BulkDeleter,
     Protocol,
 ):
-    def create(
-        self,
-        run__id: int,
-        key: str,
-        value: MetaValue,
-    ) -> RunMetaEntry:
+    def create(self, run__id: int, key: str, value: MetaValue) -> RunMetaEntry:
         """Creates a meta indicator entry for a run.
 
         Parameters
@@ -125,13 +163,14 @@ class RunMetaEntryRepository(
         ...
 
     def list(
-        self,
-        **kwargs,
+        self, join_run_index: bool = False, **kwargs: Unpack[EnumerateKwargs]
     ) -> list[RunMetaEntry]:
         r"""Lists run's meta indicator entries by specified criteria.
 
         Parameters
         ----------
+        join_run_index: bool, optional
+            Default `False`.
         \*\*kwargs: any
             Filter parameters as specified in
             `ixmp4.data.db.meta.filter.RunMetaEntryFilter`.
@@ -144,14 +183,14 @@ class RunMetaEntryRepository(
         ...
 
     def tabulate(
-        self,
-        join_run_index: bool = False,
-        **kwargs,
+        self, join_run_index: bool = False, **kwargs: Unpack[EnumerateKwargs]
     ) -> pd.DataFrame:
         r"""Tabulates run's meta indicator entries by specified criteria.
 
         Parameters
         ----------
+        join_run_index: bool, optional
+            Default `False`.
         \*\*kwargs: any
             Filter parameters as specified in
             `ixmp4.data.db.meta.filter.RunMetaEntryFilter`.
