@@ -1,10 +1,13 @@
+from datetime import datetime
 from typing import Protocol
 
 import pandas as pd
 
-from ixmp4.data import types
+# TODO Import this from typing when dropping Python 3.11
+from typing_extensions import Unpack
 
 from . import base
+from .annotations import HasIdFilter, HasNameFilter, IamcModelFilter
 from .docs import DocsRepository
 
 
@@ -13,18 +16,22 @@ class Model(base.BaseModel, Protocol):
     Unfortunately two naming conventions clash here.
     """
 
-    name: types.String
+    name: str
     "Unique name of the model."
 
-    created_at: types.DateTime
-    "Creation date/time. TODO"
-    created_by: types.String
-    "Creator. TODO"
+    created_at: datetime
+    "Creation date/time."
+    created_by: str
+    "Creator."
 
     def __str__(self) -> str:
         return f"<Model {self.id} name={self.name}>"
 
 
+class EnumerateKwargs(HasIdFilter, HasNameFilter, total=False):
+    iamc: IamcModelFilter | bool
+
+
 class ModelRepository(
     base.Creator,
     base.Retriever,
@@ -73,17 +80,14 @@ class ModelRepository(
         """
         ...
 
-    def list(
-        self,
-        *,
-        name: str | None = None,
-    ) -> list[Model]:
-        """Lists models by specified criteria.
+    def list(self, **kwargs: Unpack[EnumerateKwargs]) -> list[Model]:
+        r"""Lists models by specified criteria.
 
         Parameters
         ----------
-        name : str
-            The name of a model. If supplied only one result will be returned.
+        \*\*kwargs: any
+            Any filter parameters as specified in
+            `ixmp4.data.db.model.filter.ModelFilter`.
 
         Returns
         -------
@@ -92,18 +96,14 @@ class ModelRepository(
         """
         ...
 
-    def tabulate(
-        self,
-        *,
-        name: str | None = None,
-        **kwargs,
-    ) -> pd.DataFrame:
-        """Tabulate models by specified criteria.
+    def tabulate(self, **kwargs: Unpack[EnumerateKwargs]) -> pd.DataFrame:
+        r"""Tabulate models by specified criteria.
 
         Parameters
         ----------
-        name : str
-            The name of a model. If supplied only one result will be returned.
+        \*\*kwargs: any
+            Any filter parameters as specified in
+            `ixmp4.data.db.model.filter.ModelFilter`.
 
         Returns
         -------
@@ -115,7 +115,7 @@ class ModelRepository(
         """
         ...
 
-    def map(self, *args, **kwargs) -> dict:
+    def map(self, **kwargs: Unpack[EnumerateKwargs]) -> dict[int, str]:
         """Return a mapping of model-id to model-name.
 
         Returns
@@ -123,4 +123,4 @@ class ModelRepository(
         :class:`dict`:
             A dictionary `id` -> `name`
         """
-        return dict([(m.id, m.name) for m in self.list(*args, **kwargs)])
+        return dict([(m.id, m.name) for m in self.list(**kwargs)])
