@@ -1,27 +1,34 @@
+from datetime import datetime
 from typing import Protocol
 
 import pandas as pd
 
-from ixmp4.data import types
+# TODO Import this from typing when dropping Python 3.11
+from typing_extensions import Unpack
 
 from . import base
+from .annotations import HasIdFilter, HasNameFilter, IamcUnitFilter
 from .docs import DocsRepository
 
 
 class Unit(base.BaseModel, Protocol):
     """Unit data model."""
 
-    name: types.String
+    name: str
     "Unique name of the unit."
-    created_at: types.DateTime
-    "Creation date/time. TODO"
-    created_by: types.String
-    "Creator. TODO"
+    created_at: datetime
+    "Creation date/time."
+    created_by: str
+    "Creator."
 
     def __str__(self) -> str:
         return f"<Unit {self.id} name={self.name}>"
 
 
+class EnumerateKwargs(HasIdFilter, HasNameFilter, total=False):
+    iamc: IamcUnitFilter | bool
+
+
 class UnitRepository(
     base.Creator,
     base.Deleter,
@@ -97,7 +104,7 @@ class UnitRepository(
         except Unit.NotFound:
             return self.create(name)
 
-    def delete(self, id: int):
+    def delete(self, id: int) -> None:
         """Deletes a unit.
 
         Parameters
@@ -114,21 +121,14 @@ class UnitRepository(
         """
         ...
 
-    def list(
-        self,
-        *,
-        name: str | None = None,
-        **kwargs,
-    ) -> list[Unit]:
+    def list(self, **kwargs: Unpack[EnumerateKwargs]) -> list[Unit]:
         r"""Lists units by specified criteria.
 
         Parameters
         ----------
-        name : str
-            The name of a unit. If supplied only one result will be returned.
         \*\*kwargs: any
-            More filter parameters as specified in
-            `ixmp4.data.db.unit.filters.UnitFilter`.
+            Any filter parameters as specified in
+            `ixmp4.data.db.unit.filter.UnitFilter`.
 
         Returns
         -------
@@ -137,21 +137,14 @@ class UnitRepository(
         """
         ...
 
-    def tabulate(
-        self,
-        *,
-        name: str | None = None,
-        **kwargs,
-    ) -> pd.DataFrame:
+    def tabulate(self, **kwargs: Unpack[EnumerateKwargs]) -> pd.DataFrame:
         r"""Tabulate units by specified criteria.
 
         Parameters
         ----------
-        name : str
-            The name of a unit. If supplied only one result will be returned.
         \*\*kwargs: any
-            More filter parameters as specified in
-            `ixmp4.data.db.unit.filters.UnitFilter`.
+            Any filter parameters as specified in
+            `ixmp4.data.db.unit.filter.UnitFilter`.
 
         Returns
         -------
