@@ -1,23 +1,23 @@
 import logging
 import time
+from collections.abc import Callable, Generator, Iterable, Mapping
 from concurrent import futures
+from datetime import datetime
 from json.decoder import JSONDecodeError
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    ClassVar,
-    Generic,
-    Type,
-    TypeVar,
-)
+
+# TODO Use `type` instead of TypeAlias when dropping Python 3.11
+from typing import TYPE_CHECKING, Any, ClassVar, Generic, TypeAlias, TypeVar, cast
 
 import httpx
+import numpy as np
 import pandas as pd
 from pydantic import BaseModel as PydanticBaseModel
 from pydantic import ConfigDict, Field, GetCoreSchemaHandler
 from pydantic_core import CoreSchema, core_schema
 
+# TODO Import this from typing when dropping support for Python 3.11
+from typing_extensions import TypedDict, Unpack
+
 from ixmp4.conf import settings
 from ixmp4.core.exceptions import (
     ApiEncumbered,
@@ -26,12 +26,33 @@ from ixmp4.core.exceptions import (
     UnknownApiError,
     registry,
 )
+from ixmp4.data import abstract
 
 if TYPE_CHECKING:
     from ixmp4.data.backend.api import RestBackend
 
 logger = logging.getLogger(__name__)
 
+JsonType: TypeAlias = Mapping[
+    str,
+    Iterable[float]
+    | Iterable[int]
+    | Iterable[str]
+    | Mapping[str, Any]
+    | abstract.annotations.PrimitiveTypes
+    | None,
+]
+ParamType: TypeAlias = dict[
+    str, bool | int | str | list[int] | Iterable[int] | Mapping[str, Any] | None
+]
+_RequestParamType: TypeAlias = Mapping[
+    str,
+    abstract.annotations.PrimitiveTypes
+    | abstract.annotations.PrimitiveIterableTypes
+    | Mapping[str, Any]
+    | None,
+]
+
 
 class BaseModel(PydanticBaseModel):
     NotFound: ClassVar[type[IxmpError]]
@@ -40,26 +61,65 @@ class BaseModel(PydanticBaseModel):
     model_config = ConfigDict(from_attributes=True)
 
 
-def df_to_dict(df: pd.DataFrame) -> dict:
+class DataFrameDict(TypedDict):
+    index: list[int] | list[str]
+    columns: list[str]
+    dtypes: list[str]
+    # NOTE This is deliberately slightly out of sync with DataFrame.data below
+    # (cf positioning of int and float) to demonstrate that only the one below seems to
+    # affect our tests by causing ValidationErrors
+    data: list[
+        list[
+            abstract.annotations.PrimitiveTypes
+            | datetime
+            | dict[str, Any]
+            | list[float]
+            | list[int]
+            | list[str]
+            | None
+        ]
+    ]
+
+
+def df_to_dict(df: pd.DataFrame) -> DataFrameDict:
     columns = []
     dtypes = []
     for c in df.columns:
         columns.append(c)
         dtypes.append(df[c].dtype.name)
 
-    return {
-        "index": df.index.to_list(),
-        "columns": columns,
-        "dtypes": dtypes,
-        "data": df.values.tolist(),
-    }
+    return DataFrameDict(
+        index=df.index.to_list(),
+        columns=columns,
+        dtypes=dtypes,
+        data=df.values.tolist(),
+    )
 
 
 class DataFrame(PydanticBaseModel):
-    index: list | None = Field(None)
+    index: list[int] | list[str] | None = Field(None)
     columns: list[str] | None
     dtypes: list[str] | None
-    data: list | None
+    # TODO The order is important here at the moment, in particular having int before
+    # float! This should likely not be the case, but using StrictInt and StrictFloat
+    # from pydantic only created even more errors.
+    data: (
+        list[
+            list[
+                bool
+                | datetime
+                | int
+                | float
+                | str
+                | dict[str, Any]
+                | list[float]
+                | list[int]
+                | list[str]
+                | None
+            ]
+        ]
+        | None
+    )
 
     model_config = ConfigDict(json_encoders={pd.Timestamp: lambda x: x.isoformat()})
 
@@ -68,14 +128,10 @@ class DataFrame(PydanticBaseModel):
         cls, source_type: Any, handler: GetCoreSchemaHandler
     ) -> CoreSchema:
         return core_schema.no_info_before_validator_function(cls.validate, handler(cls))
-        # yield cls.validate
 
     @classmethod
-    def validate(cls, df: pd.DataFrame | dict):
-        if isinstance(df, pd.DataFrame):
-            return df_to_dict(df)
-        else:
-            return df
+    def validate(cls, df: pd.DataFrame | DataFrameDict) -> DataFrameDict:
+        return df_to_dict(df) if isinstance(df, pd.DataFrame) else df
 
     def to_pandas(self) -> pd.DataFrame:
         df = pd.DataFrame(
@@ -86,27 +142,37 @@ class DataFrame(PydanticBaseModel):
         if self.columns and self.dtypes:
             for c, dt in zip(self.columns, self.dtypes):
                 # there seems to be a type incompatbility between StrDtypeArg and str
-                df[c] = df[c].astype(dt)  # type: ignore
+                df[c] = df[c].astype(dt)  # type: ignore[call-overload]
         return df
 
 
+class _RequestKwargs(TypedDict, total=False):
+    params: _RequestParamType | None
+    json: JsonType | None
+    max_retries: int
+
+
+class RequestKwargs(TypedDict, total=False):
+    content: str
+
+
 ModelType = TypeVar("ModelType", bound=BaseModel)
 
 
 class BaseRepository(Generic[ModelType]):
-    model_class: Type[ModelType]
+    model_class: type[ModelType]
     prefix: ClassVar[str]
     enumeration_method: str = "PATCH"
 
     backend: "RestBackend"
 
-    def __init__(self, backend: "RestBackend", *args, **kwargs) -> None:
+    def __init__(self, backend: "RestBackend") -> None:
         self.backend = backend
 
-    def sanitize_params(self, params: dict):
+    def sanitize_params(self, params: Mapping[str, Any]) -> dict[str, Any]:
         return {k: params[k] for k in params if params[k] is not None}
 
-    def get_remote_exception(self, res: httpx.Response, status_code: int):
+    def get_remote_exception(self, res: httpx.Response, status_code: int) -> IxmpError:
         try:
             json = res.json()
         except (ValueError, JSONDecodeError):
@@ -130,11 +196,11 @@ class BaseRepository(Generic[ModelType]):
         self,
         method: str,
         path: str,
-        params: dict | None = None,
-        json: dict | None = None,
+        params: _RequestParamType | None = None,
+        json: JsonType | None = None,
         max_retries: int = settings.client_max_request_retries,
-        **kwargs,
-    ) -> dict | list | None:
+        **kwargs: Unpack[RequestKwargs],
+    ) -> dict[str, Any] | list[Any] | None:
         """Sends a request and handles potential error responses.
         Re-raises a remote `IxmpError` if thrown and transferred from the backend.
         Handles read timeouts and rate limiting responses via retries with backoffs.
@@ -142,7 +208,7 @@ class BaseRepository(Generic[ModelType]):
         but has a status code less than 300.
         """
 
-        def retry(max_retries=max_retries) -> dict | list | None:
+        def retry(max_retries: int = max_retries) -> dict[str, Any] | list[Any] | None:
             if max_retries == 0:
                 logger.error(f"API Encumbered: '{self.backend.info.dsn}'")
                 raise ApiEncumbered(
@@ -186,8 +252,8 @@ class BaseRepository(Generic[ModelType]):
     def _handle_response(
         self,
         res: httpx.Response,
-        retry: Callable[..., dict | list | None],
-    ) -> dict | list | None:
+        retry: Callable[..., dict[str, Any] | list[Any] | None],
+    ) -> dict[str, Any] | list[Any] | None:
         if res.status_code in [
             429,  # Too Many Requests
             420,  # Enhance Your Calm
@@ -203,7 +269,9 @@ class BaseRepository(Generic[ModelType]):
             raise self.get_remote_exception(res, res.status_code)
         else:
             try:
-                return res.json()
+                # res.json just returns Any...
+                json_decoded: dict[str, Any] | list[Any] = res.json()
+                return json_decoded
             except JSONDecodeError:
                 if res.status_code < 300 and res.text == "":
                     return None
@@ -211,63 +279,69 @@ class BaseRepository(Generic[ModelType]):
                 pass
             raise UnknownApiError(res.text)
 
-    def _get_by_id(self, id: int, *args, **kwargs) -> dict[str, Any]:
+    def _get_by_id(self, id: int) -> dict[str, Any]:
         # we can assume this type on create endpoints
-        return self._request("GET", self.prefix + str(id) + "/", **kwargs)  # type: ignore
+        return self._request("GET", self.prefix + str(id) + "/")  # type: ignore[return-value]
 
     def _request_enumeration(
         self,
         table: bool = False,
-        params: dict | None = None,
-        json: dict | None = None,
-    ):
+        params: ParamType | None = None,
+        json: JsonType | None = None,
+        path: str = "",
+    ) -> dict[str, Any] | list[Any]:
         """Convenience method for requests to the enumeration endpoint."""
         if params is None:
             params = {}
 
+        # See https://github.com/iiasa/ixmp4/pull/129#discussion_r1841829519 for why we
+        # are keeping this assumption
+        # we can assume these types on enumeration endpoints
         return self._request(
             self.enumeration_method,
-            self.prefix,
+            self.prefix + path,
             params={**params, "table": table},
             json=json,
-        )
+        )  # type: ignore[return-value]
 
     def _dispatch_pagination_requests(
         self,
         total: int,
         start: int,
         limit: int,
-        params: dict | None,
-        json: dict | None,
-    ) -> list[list | dict]:
+        params: ParamType | None,
+        json: JsonType | None,
+        path: str = "",
+    ) -> list[list[Any]] | list[dict[str, Any]]:
         """Uses the backends executor to send many pagination requests concurrently."""
-        requests: list[futures.Future] = []
+        requests: list[futures.Future[dict[str, Any]]] = []
         for req_offset in range(start, total, limit):
-            if params is not None:
-                req_params = params.copy()
-            else:
-                req_params = {}
+            req_params = params.copy() if params is not None else {}
 
             req_params.update({"limit": limit, "offset": req_offset})
-            futu = self.backend.executor.submit(
-                self._request,
+            # Based on usage below, we seem to rely on self._request always returning a
+            # dict[str, Any] here
+            futu: futures.Future[dict[str, Any]] = self.backend.executor.submit(
+                self._request,  # type: ignore [arg-type]
                 self.enumeration_method,
-                self.prefix,
+                self.prefix + path,
                 params=req_params,
                 json=json,
             )
             requests.append(futu)
         results = futures.wait(requests)
         responses = [f.result() for f in results.done]
+        # This seems to imply that type(responses) == list[dict[str, Any]]
         return [r.pop("results") for r in responses]
 
     def _handle_pagination(
         self,
-        data: dict,
+        data: dict[str, Any],
         table: bool = False,
-        params: dict | None = None,
-        json: dict | None = None,
-    ) -> list[list] | list[dict]:
+        params: ParamType | None = None,
+        json: JsonType | None = None,
+        path: str = "",
+    ) -> list[list[Any]] | list[dict[str, Any]]:
         """Handles paginated response and sends subsequent requests if necessary.
         Returns aggregated pages as a list."""
 
@@ -284,56 +358,86 @@ class BaseRepository(Generic[ModelType]):
             return [data.pop("results")]
         else:
             results = self._dispatch_pagination_requests(
-                total, offset + limit, limit, params, json
+                total, offset + limit, limit, params, json, path=path
             )
             return [data.pop("results")] + results
 
     def _list(
-        self, params: dict | None = None, json: dict | None = None, **kwargs
+        self,
+        params: ParamType | None = None,
+        json: JsonType | None = None,
+        path: str = "",
     ) -> list[ModelType]:
         data = self._request_enumeration(params=params, table=False, json=json)
         if isinstance(data, dict):
             # we can assume this type on list endpoints
-            pages: list[list] = self._handle_pagination(
+            pages: list[list[Any]] = self._handle_pagination(
                 data, table=False, params=params, json=json
-            )  # type: ignore
+            )  # type: ignore[assignment]
             results = [i for page in pages for i in page]
         else:
             results = data
         return [self.model_class(**i) for i in results]
 
     def _tabulate(
-        self, params: dict | None = {}, json: dict | None = None, **kwargs
+        self,
+        params: ParamType | None = {},
+        json: JsonType | None = None,
+        path: str = "",
     ) -> pd.DataFrame:
-        data = self._request_enumeration(table=True, params=params, json=json)
+        # we can assume this type on table endpoints
+        data: dict[str, Any] = self._request_enumeration(
+            table=True, params=params, json=json, path=path
+        )  # type: ignore[assignment]
         pagination = data.get("pagination", None)
         if pagination is not None:
             # we can assume this type on table endpoints
-            pages: list[dict] = self._handle_pagination(
-                data,
-                table=True,
-                params=params,
-                json=json,
-            )  # type: ignore
+            pages: list[dict[str, Any]] = self._handle_pagination(
+                data, table=True, params=params, json=json, path=path
+            )  # type: ignore[assignment]
             dfs = [DataFrame(**page).to_pandas() for page in pages]
-            return pd.concat(dfs)
+            return pd.concat(dfs).replace({np.nan: None})
         else:
-            return DataFrame(**data).to_pandas()
+            return DataFrame(**data).to_pandas().replace({np.nan: None})
 
-    def _create(self, *args, **kwargs) -> dict[str, Any]:
+    def _create(
+        self, *args: Unpack[tuple[str]], **kwargs: Unpack[_RequestKwargs]
+    ) -> dict[str, Any]:
         # we can assume this type on create endpoints
-        return self._request("POST", *args, **kwargs)  # type: ignore
+        return self._request("POST", *args, **kwargs)  # type: ignore[return-value]
 
-    def _delete(self, id: int):
+    def _delete(self, id: int) -> None:
         self._request("DELETE", f"{self.prefix}{str(id)}/")
 
 
+class GetKwargs(TypedDict, total=False):
+    dimension_id: int
+    run_ids: list[int]
+    parameters: Mapping[str, Any]
+    name: str
+    run_id: int
+    key: str
+    model: dict[str, str]
+    scenario: dict[str, str]
+    version: int
+    default_only: bool
+    is_default: bool | None
+
+
 class Retriever(BaseRepository[ModelType]):
-    def get(self, **kwargs) -> ModelType:
-        if self.enumeration_method == "GET":
-            list_ = self._list(params=kwargs)
-        else:
-            list_ = self._list(json=kwargs)
+    def get(self, **kwargs: Unpack[GetKwargs]) -> ModelType:
+        _kwargs = cast(
+            dict[
+                str,
+                bool | int | str | list[int] | Iterable[int] | Mapping[str, Any] | None,
+            ],
+            kwargs,
+        )
+        list_ = (
+            self._list(params=_kwargs)
+            if self.enumeration_method == "GET"
+            else self._list(json=_kwargs)
+        )
 
         try:
             [obj] = list_
@@ -345,62 +449,78 @@ class Retriever(BaseRepository[ModelType]):
 
 
 class Creator(BaseRepository[ModelType]):
-    def create(self, **kwargs) -> ModelType:
-        res = self._create(
-            self.prefix,
-            json=kwargs,
-        )
+    def create(
+        self,
+        **kwargs: int
+        | str
+        | Mapping[str, Any]
+        | abstract.MetaValue
+        | list[str]
+        | float
+        | None,
+    ) -> ModelType:
+        res = self._create(self.prefix, json=kwargs)
         return self.model_class(**res)
 
 
 class Deleter(BaseRepository[ModelType]):
-    def delete(self, id: int):
+    def delete(self, id: int) -> None:
         self._delete(id)
 
 
+class ListKwargs(TypedDict, total=False):
+    run_id: int
+    name: str
+
+
 class Lister(BaseRepository[ModelType]):
-    def list(self, *args, **kwargs) -> list[ModelType]:
-        return self._list(json=kwargs)
+    def list(self, **kwargs: Unpack[ListKwargs]) -> list[ModelType]:
+        return self._list(json=kwargs)  # type: ignore[arg-type]
 
 
 class Tabulator(BaseRepository[ModelType]):
-    def tabulate(self, *args, **kwargs) -> pd.DataFrame:
+    def tabulate(self, **kwargs: Any) -> pd.DataFrame:
         return self._tabulate(json=kwargs)
 
 
 class Enumerator(Lister[ModelType], Tabulator[ModelType]):
     def enumerate(
-        self, *args, table: bool = False, **kwargs
+        self, table: bool = False, **kwargs: Any
     ) -> list[ModelType] | pd.DataFrame:
-        if table:
-            return self.tabulate(*args, **kwargs)
-        else:
-            return self.list(*args, **kwargs)
+        return self.tabulate(**kwargs) if table else self.list(**kwargs)
 
 
 class BulkOperator(BaseRepository[ModelType]):
-    def yield_chunks(self, df: pd.DataFrame, chunk_size: int):
+    def yield_chunks(
+        self, df: pd.DataFrame, chunk_size: int
+    ) -> Generator[pd.DataFrame, Any, None]:
         for _, chunk in df.groupby(df.index // chunk_size):
             yield chunk
 
 
+class BulkUpsertKwargs(TypedDict, total=False):
+    create_related: bool
+
+
 class BulkUpserter(BulkOperator[ModelType]):
     def bulk_upsert(
         self,
         df: pd.DataFrame,
         chunk_size: int = settings.client_default_upload_chunk_size,
-        **kwargs,
-    ):
+        **kwargs: Unpack[BulkUpsertKwargs],
+    ) -> None:
         for chunk in self.yield_chunks(df, chunk_size):
             self.bulk_upsert_chunk(chunk, **kwargs)
 
-    def bulk_upsert_chunk(self, df: pd.DataFrame, **kwargs) -> None:
+    def bulk_upsert_chunk(
+        self, df: pd.DataFrame, **kwargs: Unpack[BulkUpsertKwargs]
+    ) -> None:
         dict_ = df_to_dict(df)
         json_ = DataFrame(**dict_).model_dump_json()
         self._request(
             "POST",
             self.prefix + "bulk/",
-            params=kwargs,
+            params=cast(dict[str, bool | None], kwargs),
             content=json_,
         )
 
@@ -410,12 +530,14 @@ class BulkDeleter(BulkOperator[ModelType]):
         self,
         df: pd.DataFrame,
         chunk_size: int = settings.client_default_upload_chunk_size,
-        **kwargs,
-    ):
+        # NOTE nothing in our code base supplies kwargs here
+        **kwargs: Any,
+    ) -> None:
         for chunk in self.yield_chunks(df, chunk_size):
             self.bulk_delete_chunk(chunk, **kwargs)
 
-    def bulk_delete_chunk(self, df: pd.DataFrame, **kwargs) -> None:
+    # NOTE this only gets kwargs from bulk_delete()
+    def bulk_delete_chunk(self, df: pd.DataFrame, **kwargs: Any) -> None:
         dict_ = df_to_dict(df)
         json_ = DataFrame(**dict_).model_dump_json()
         self._request(
