@@ -1,9 +1,15 @@
 from datetime import datetime
-from typing import ClassVar
+from typing import TYPE_CHECKING, ClassVar, cast
+
+if TYPE_CHECKING:
+    from ixmp4.data.backend.api import RestBackend
 
 import pandas as pd
 from pydantic import Field
 
+# TODO Import this from typing when dropping support for Python 3.11
+from typing_extensions import Unpack
+
 from ixmp4.data import abstract
 
 from . import base
@@ -31,27 +37,30 @@ class ModelRepository(
     model_class = Model
     prefix = "models/"
 
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
+    def __init__(self, *args: Unpack[tuple["RestBackend"]]) -> None:
+        super().__init__(*args)
         self.docs = ModelDocsRepository(self.backend)
 
-    def create(
-        self,
-        name: str,
-    ) -> Model:
+    def create(self, name: str) -> Model:
         return super().create(name=name)
 
     def get(self, name: str) -> Model:
         return super().get(name=name)
 
-    def enumerate(self, **kwargs) -> list[Model] | pd.DataFrame:
+    def enumerate(
+        self, **kwargs: Unpack[abstract.model.EnumerateKwargs]
+    ) -> list[Model] | pd.DataFrame:
         return super().enumerate(**kwargs)
 
-    def list(self, **kwargs) -> list[Model]:
-        return super()._list(json=kwargs)
+    def list(self, **kwargs: Unpack[abstract.model.EnumerateKwargs]) -> list[Model]:
+        json = cast(abstract.annotations.IamcObjectFilterAlias, kwargs)
+        return super()._list(json=json)
 
-    def tabulate(self, **kwargs) -> pd.DataFrame:
-        return super()._tabulate(json=kwargs)
+    def tabulate(
+        self, **kwargs: Unpack[abstract.model.EnumerateKwargs]
+    ) -> pd.DataFrame:
+        json = cast(abstract.annotations.IamcObjectFilterAlias, kwargs)
+        return super()._tabulate(json=json)
 
 
 class ModelDocsRepository(DocsRepository):
