@@ -1,9 +1,15 @@
 from datetime import datetime
-from typing import ClassVar
+from typing import TYPE_CHECKING, ClassVar, cast
+
+if TYPE_CHECKING:
+    from ixmp4.data.backend.api import RestBackend
 
 import pandas as pd
 from pydantic import Field
 
+# TODO Import this from typing when dropping support for Python 3.11
+from typing_extensions import Unpack
+
 from ixmp4.data import abstract
 
 from . import base
@@ -44,15 +50,11 @@ class RegionRepository(
     model_class = Region
     prefix = "regions/"
 
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
+    def __init__(self, *args: Unpack[tuple["RestBackend"]]) -> None:
+        super().__init__(*args)
         self.docs = RegionDocsRepository(self.backend)
 
-    def create(
-        self,
-        name: str,
-        hierarchy: str,
-    ) -> Region:
+    def create(self, name: str, hierarchy: str) -> Region:
         return super().create(name=name, hierarchy=hierarchy)
 
     def delete(self, id: int) -> None:
@@ -61,11 +63,17 @@ class RegionRepository(
     def get(self, name: str) -> Region:
         return super().get(name=name)
 
-    def enumerate(self, **kwargs) -> list[Region] | pd.DataFrame:
+    def enumerate(
+        self, **kwargs: Unpack[abstract.region.EnumerateKwargs]
+    ) -> list[Region] | pd.DataFrame:
         return super().enumerate(**kwargs)
 
-    def list(self, **kwargs) -> list[Region]:
-        return super()._list(json=kwargs)
+    def list(self, **kwargs: Unpack[abstract.region.EnumerateKwargs]) -> list[Region]:
+        json = cast(abstract.annotations.IamcObjectFilterAlias, kwargs)
+        return super()._list(json=json)
 
-    def tabulate(self, **kwargs) -> pd.DataFrame:
-        return super()._tabulate(json=kwargs)
+    def tabulate(
+        self, **kwargs: Unpack[abstract.region.EnumerateKwargs]
+    ) -> pd.DataFrame:
+        json = cast(abstract.annotations.IamcObjectFilterAlias, kwargs)
+        return super()._tabulate(json=json)
