@@ -1,9 +1,15 @@
 from datetime import datetime
-from typing import ClassVar
+from typing import TYPE_CHECKING, ClassVar, cast
+
+if TYPE_CHECKING:
+    from ixmp4.data.backend.api import RestBackend
 
 import pandas as pd
 from pydantic import Field
 
+# TODO Import this from typing when dropping support for Python 3.11
+from typing_extensions import Unpack
+
 from ixmp4.data import abstract
 
 from . import base
@@ -35,24 +41,29 @@ class ScenarioRepository(
     model_class = Scenario
     prefix = "scenarios/"
 
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
+    def __init__(self, *args: Unpack[tuple["RestBackend"]]) -> None:
+        super().__init__(*args)
         self.docs = ScenarioDocsRepository(self.backend)
 
-    def create(
-        self,
-        name: str,
-    ) -> Scenario:
+    def create(self, name: str) -> Scenario:
         return super().create(name=name)
 
     def get(self, name: str) -> Scenario:
         return super().get(name=name)
 
-    def enumerate(self, **kwargs) -> list[Scenario] | pd.DataFrame:
+    def enumerate(
+        self, **kwargs: Unpack[abstract.scenario.EnumerateKwargs]
+    ) -> list[Scenario] | pd.DataFrame:
         return super().enumerate(**kwargs)
 
-    def list(self, **kwargs) -> list[Scenario]:
-        return super()._list(json=kwargs)
+    def list(
+        self, **kwargs: Unpack[abstract.scenario.EnumerateKwargs]
+    ) -> list[Scenario]:
+        json = cast(abstract.annotations.IamcObjectFilterAlias, kwargs)
+        return super()._list(json=json)
 
-    def tabulate(self, **kwargs) -> pd.DataFrame:
-        return super()._tabulate(json=kwargs)
+    def tabulate(
+        self, **kwargs: Unpack[abstract.scenario.EnumerateKwargs]
+    ) -> pd.DataFrame:
+        json = cast(abstract.annotations.IamcObjectFilterAlias, kwargs)
+        return super()._tabulate(json=json)
