@@ -3,6 +3,7 @@ This module contains sqlalchemy database models and repositories.
 """
 
 from .base import BaseModel, BaseRepository
+from .checkpoint import Checkpoint, CheckpointRepository
 from .docs import BaseDocsRepository, docs_model
 from .iamc import (  # AnnualDataPoint,; SubAnnualDataPoint,; CategoricalDataPoint,
     DataPoint,
@@ -18,8 +19,6 @@ from .iamc import (  # AnnualDataPoint,; SubAnnualDataPoint,; CategoricalDataPoi
 from .meta import RunMetaEntry, RunMetaEntryRepository
 from .model import Model, ModelRepository
 from .optimization import (
-    Column,
-    ColumnRepository,
     Equation,
     EquationRepository,
     IndexSet,
@@ -31,11 +30,10 @@ from .optimization import (
     Table,
     TableRepository,
 )
-
-# TODO for PR: avoiding name conflict here Is that okay?
 from .optimization import Variable as OptimizationVariable
 from .optimization import VariableRepository as OptimizationVariableRepository
 from .region import Region, RegionRepository
 from .run import Run, RunRepository
 from .scenario import Scenario, ScenarioRepository
 from .unit import Unit, UnitRepository
+from .versions import TransactionRepository
