@@ -1,9 +1,8 @@
 from typing import ClassVar
 
-from sqlalchemy.ext.declarative import declared_attr
-
 from ixmp4 import db
 from ixmp4.data import abstract
+from ixmp4.data.db import versions
 
 from .. import base
 
@@ -19,14 +18,12 @@ class DataPoint(base.BaseModel):
 
     updateable_columns = ["value"]
 
-    @declared_attr
-    def time_series__id(cls):
-        return db.Column(
-            db.Integer,
-            db.ForeignKey("iamc_timeseries.id"),
-            nullable=False,
-            index=True,
-        )
+    time_series__id: db.MappedColumn[int] = db.Column(
+        db.Integer,
+        db.ForeignKey("iamc_timeseries.id"),
+        nullable=False,
+        index=True,
+    )
 
     value = db.Column(db.Float)
 
@@ -38,9 +35,7 @@ class DataPoint(base.BaseModel):
 
 
 class UniversalDataPoint(DataPoint):
-    @declared_attr.directive
-    def __tablename__(cls) -> str:
-        return "iamc_datapoint_universal"
+    __tablename__ = "iamc_datapoint_universal"
 
     __table_args__ = (
         db.UniqueConstraint("time_series__id", "step_year", "step_category"),
@@ -48,5 +43,28 @@ class UniversalDataPoint(DataPoint):
     )
 
 
-def get_datapoint_model(session) -> type[DataPoint]:
+def get_datapoint_model(session: db.Session) -> type[DataPoint]:
     return UniversalDataPoint
+
+
+class UniversalDataPointVersion(versions.DefaultVersionModel):
+    __tablename__ = "iamc_datapoint_universal_version"
+
+    value = db.Column(db.Float)
+
+    time_series__id: db.MappedColumn[int] = db.Column(
+        db.Integer,
+        nullable=False,
+        index=True,
+    )
+
+    type = db.Column(db.String(255), nullable=False, index=True)
+
+    step_category = db.Column(db.String(1023), index=True)
+    step_year = db.Column(db.Integer, index=True)
+    step_datetime = db.Column(db.DateTime, index=True)
+
+
+version_triggers = versions.PostgresVersionTriggers(
+    UniversalDataPoint.__table__, UniversalDataPointVersion.__table__
+)
