@@ -1,7 +1,8 @@
-from typing import Mapping
+from collections.abc import Mapping
 
 from ixmp4 import db
 from ixmp4.data import types
+from ixmp4.data.db import versions
 from ixmp4.data.db.iamc.measurand import Measurand
 from ixmp4.data.db.region import Region
 from ixmp4.data.db.timeseries import TimeSeries as BaseTimeSeries
@@ -10,6 +11,8 @@ from .. import base
 
 
 class TimeSeries(BaseTimeSeries, base.BaseModel):
+    __tablename__ = "iamc_timeseries"
+
     __table_args__ = (db.UniqueConstraint("run__id", "region__id", "measurand__id"),)
 
     region__id: types.Integer = db.Column(
@@ -27,9 +30,21 @@ class TimeSeries(BaseTimeSeries, base.BaseModel):
     )
 
     @property
-    def parameters(self) -> Mapping:
+    def parameters(self) -> Mapping[str, str]:
         return {
             "region": self.region.name,
             "unit": self.measurand.unit.name,
             "variable": self.measurand.variable.name,
         }
+
+
+class TimeSeriesVersion(versions.DefaultVersionModel):
+    __tablename__ = "iamc_timeseries_version"
+    region__id: types.Integer = db.Column(nullable=False, index=True)
+    measurand__id: types.Integer = db.Column(nullable=False, index=True)
+    run__id: types.Integer = db.Column(nullable=False, index=True)
+
+
+version_triggers = versions.PostgresVersionTriggers(
+    TimeSeries.__table__, TimeSeriesVersion.__table__
+)
