@@ -2,6 +2,8 @@ from typing import ClassVar
 
 from ixmp4.core.exceptions import IxmpError
 from ixmp4.data import types
+from ixmp4.data.abstract.annotations import HasIdFilter, HasNameFilter, HasRunIdFilter
+from ixmp4.db.filters import BaseFilter
 
 from .. import mixins
 from ..base import BaseModel as RootBaseModel
@@ -23,6 +25,9 @@ class BaseModel(RootBaseModel, mixins.HasCreationInfo):
     DataInvalid: ClassVar[type[IxmpError]]
 
     __abstract__ = True
-    table_prefix = "optimization_"
 
     name: types.Name
+
+
+class EnumerateKwargs(HasIdFilter, HasNameFilter, HasRunIdFilter, total=False):
+    _filter: BaseFilter
