@@ -11,6 +11,8 @@ from .. import base
 
 
 class IndexSet(base.BaseModel):
+    __tablename__ = "optimization_indexset"
+
     NotFound: ClassVar = abstract.IndexSet.NotFound
     NotUnique: ClassVar = abstract.IndexSet.NotUnique
     DataInvalid: ClassVar = OptimizationDataValidationError
@@ -19,11 +21,14 @@ class IndexSet(base.BaseModel):
     _data_type: types.OptimizationDataType
 
     _data: types.Mapped[list["IndexSetData"]] = db.relationship(
-        back_populates="indexset"
+        back_populates="indexset",
+        order_by="IndexSetData.id",
+        cascade="all, delete",
+        passive_deletes=True,
     )
 
     @property
-    def data(self) -> list[float | int | str]:
+    def data(self) -> list[float] | list[int] | list[str]:
         return (
             []
             if self._data_type is None
@@ -31,7 +36,7 @@ class IndexSet(base.BaseModel):
         )
 
     @data.setter
-    def data(self, value: list[float | int | str]) -> None:
+    def data(self, value: list[float] | list[int] | list[str]) -> None:
         return None
 
     run__id: types.RunId
@@ -40,10 +45,10 @@ class IndexSet(base.BaseModel):
 
 
 class IndexSetData(base.RootBaseModel):
-    table_prefix = "optimization_"
+    __tablename__ = "optimization_indexsetdata"
 
     indexset: types.Mapped["IndexSet"] = db.relationship(back_populates="_data")
-    indexset__id: types.IndexSetId
+    indexset__id: types.IndexSet__Id
     value: types.String = db.Column(db.String, nullable=False)
 
     __table_args__ = (db.UniqueConstraint("indexset__id", "value"),)
