@@ -1,4 +1,3 @@
-# flake8: noqa
 from .docs import RegionDocsRepository
-from .model import Region
+from .model import Region, RegionVersion
 from .repository import RegionRepository
