@@ -1,5 +1,3 @@
-# flake8: noqa
-
 from .docs import UnitDocs, UnitDocsRepository
-from .model import Unit
+from .model import Unit, UnitVersion
 from .repository import UnitRepository
