@@ -1,3 +1,6 @@
+from typing import Any
+
+import numpy as np
 import pandas as pd
 
 
@@ -6,7 +9,7 @@ def map_existing(
     existing_df: pd.DataFrame,
     join_on: tuple[str, str],
     map: tuple[str, str],
-):
+) -> tuple[pd.DataFrame, np.ndarray[Any, np.dtype[np.str_]]]:
     _, join_to = join_on
     _, map_to = map
     existing_df = existing_df.rename(columns=dict([join_on, map]))[[join_to, map_to]]
