@@ -1,5 +1,5 @@
 from datetime import datetime
-from typing import Any, Literal
+from typing import Literal
 
 from sqlalchemy.orm import Mapped as Mapped
 
@@ -8,13 +8,19 @@ from ixmp4 import db
 Boolean = Mapped[bool]
 DateTime = Mapped[datetime]
 Float = Mapped[float]
+EquationId = Mapped[db.EquationIdType]
 IndexSetId = Mapped[db.IndexSetIdType]
+IndexSet__Id = Mapped[db.IndexSet__IdType]
+Id = Mapped[int]
 Integer = Mapped[int]
-OptimizationDataList = Mapped[list[float | int | str]]
-JsonDict = Mapped[dict[str, Any]]
-OptimizationDataType = Mapped[Literal["float", "int", "str"] | None]
-String = Mapped[str]
+JsonDict = Mapped[dict[str, list[float] | list[int] | list[str]]]
 Name = Mapped[db.NameType]
-UniqueName = Mapped[db.UniqueNameType]
+OptimizationDataList = Mapped[list[float] | list[int] | list[str]]
+OptimizationDataType = Mapped[Literal["float", "int", "str"] | None]
+OptimizationVariableType = Mapped[db.OptimizationVariableIdType]
+ParameterId = Mapped[db.ParameterIdType]
 RunId = Mapped[db.RunIdType]
+String = Mapped[str]
+TableId = Mapped[db.TableIdType]
+UniqueName = Mapped[db.UniqueNameType]
 Username = Mapped[db.UsernameType]
