@@ -1,7 +1,23 @@
-from logging.config import fileConfig
+"""
+When making schema migrations for versioned tables
+you need to remember to call `sync_trigger` in order
+to keep the version trigger up-to-date.
+
+```
+from alembic import context
+
+if not context.is_offline_mode():
+    conn = context.get_bind()
+    sync_trigger(conn, "my_table_version")
+```
+"""
+
+from typing import Literal
 
 from alembic import context
-from sqlalchemy import create_engine
+from sqlalchemy import Engine, create_engine
+from sqlalchemy.orm import configure_mappers
+from sqlalchemy.schema import SchemaItem
 
 from ixmp4.conf import settings
 from ixmp4.data.db import BaseModel
@@ -12,10 +28,7 @@ config = context.config
 dsn = config.get_main_option("sqlalchemy.url", settings.migration_db_uri)
 dsn = dsn.replace("postgresql://", "postgresql+psycopg://")
 
-# Interpret the config file for Python logging.
-# This line sets up loggers basically.
-assert config.config_file_name is not None
-fileConfig(config.config_file_name)
+configure_mappers()
 
 target_metadata = BaseModel.metadata
 
@@ -25,18 +38,27 @@ target_metadata = BaseModel.metadata
 # ... etc.
 
 
-def include_object(object, name, type_, reflected, compare_to):
-    if (
-        type_ == "column"
-        and not reflected
-        and object.info.get("skip_autogenerate", False)
-    ):
+def include_object(
+    obj: SchemaItem,
+    name: str | None,
+    type_: Literal[
+        "schema",
+        "table",
+        "column",
+        "index",
+        "unique_constraint",
+        "foreign_key_constraint",
+    ],
+    reflected: bool,
+    compare_to: SchemaItem | None,
+) -> bool:
+    if type_ == "column" and not reflected and obj.info.get("skip_autogenerate", False):
         return False
     else:
         return True
 
 
-def run_migrations_offline():
+def run_migrations_offline() -> None:
     """Run migrations in 'offline' mode.
 
     This configures the context with just a URL
@@ -62,18 +84,20 @@ def run_migrations_offline():
         context.run_migrations()
 
 
-def run_migrations_online():
+def run_migrations_online() -> None:
     """Run migrations in 'online' mode.
 
     In this scenario we need to create an Engine
     and associate a connection with the context.
 
     """
-    connectable = create_engine(dsn)
+    engine: Engine | None = context.config.attributes.get("connection", None)
+    if engine is None:
+        engine = create_engine(dsn, max_identifier_length=63)
 
-    with connectable.connect() as connection:
+    with engine.connect() as conn:
         context.configure(
-            connection=connection,
+            connection=conn,
             target_metadata=target_metadata,
             compare_type=True,
             render_as_batch=True,
