@@ -1,13 +1,17 @@
 import logging
 import time
+from collections.abc import Awaitable, Callable
 
+from fastapi import Request, Response
 from starlette.middleware.base import BaseHTTPMiddleware
 
 logger = logging.getLogger(__name__)
 
 
 class RequestTimeLoggerMiddleware(BaseHTTPMiddleware):
-    async def dispatch(self, request, call_next):
+    async def dispatch(
+        self, request: Request, call_next: Callable[[Request], Awaitable[Response]]
+    ) -> Response:
         start_time = time.time()
         response = await call_next(request)
         process_time = time.time() - start_time
@@ -16,7 +20,9 @@ class RequestTimeLoggerMiddleware(BaseHTTPMiddleware):
 
 
 class RequestSizeLoggerMiddleware(BaseHTTPMiddleware):
-    async def dispatch(self, request, call_next):
+    async def dispatch(
+        self, request: Request, call_next: Callable[[Request], Awaitable[Response]]
+    ) -> Response:
         body = await request.body()
         logger.debug(f"Request body size: {len(body)} bytes.")
         return await call_next(request)
